//
//  MWGeometry.h
//  Malkinware
//
//  Created by Jim McGowan on 07/11/2009.
//  Copyright 2009 Jim McGowan. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "MWVector.h"


/* Rectangle Resizing Deltas */

typedef struct _MWRectResizeDelta {
    CGFloat left;
	CGFloat right;
	CGFloat top;
	CGFloat bottom;
} MWRectResizeDelta;

#define MWZeroRectResizeDelta MWMakeRectResizeDelta(0.0, 0.0, 0.0, 0.0)

MWRectResizeDelta MWMakeRectResizeDelta(CGFloat left, CGFloat right, CGFloat top, CGFloat bottom);
MWRectResizeDelta deltaBetweenRects(NSRect rect1, NSRect rect2);
BOOL MWEqualRectResizeDeltas(MWRectResizeDelta delta1, MWRectResizeDelta delta2);
NSPoint originDeltaOfRectResizeDelta(MWRectResizeDelta rectResizeDelta);
NSRect rectByApplyingDelta(NSRect rect, MWRectResizeDelta delta);
NSString *NSStringFromMWRectResizeDelta(MWRectResizeDelta delta);


/* Getting Points from Rectangles */

NSPoint bottomLeftPoint(NSRect rect);
NSPoint bottomRightPoint(NSRect rect);
NSPoint topLeftPoint(NSRect rect);
NSPoint topRightPoint(NSRect rect);

NSPoint midLeftPoint(NSRect rect);
NSPoint midRightPoint(NSRect rect);
NSPoint centerTopPoint(NSRect rect);
NSPoint centerBottomPoint(NSRect rect);
NSPoint midCenterPoint(NSRect);


/* Manipulating Rectangles, Points & Sizes */

NSRect centerRectInsideRect(NSRect internalRect, NSRect enclosingRect, BOOL *internalRectExceedsBounds); // BOOL pointer set on output
NSRect nonNegativeRect(NSRect aRect);	//creates rect with same area is aRect, but with non -ve W & H
NSRect flippedRect(NSRect rect, NSRect enclosingBounds);
NSRect nearestIntegralRect(NSRect rect);	// Unlike NSIntegralRectWithOptions(), this method can handle rects with -ve W & H
NSRect rectByAdjustingOrigin(NSRect rect, NSPoint originDelta);
NSRect constrainRectToInsideRect(NSRect internalRect, NSRect enclosingRect, BOOL *internalRectExceedsBounds); // BOOL pointer set on output

NSPoint constrainPointToInsideRect(NSPoint point, NSRect rect);
NSPoint constrainPointToOutsideRect(NSPoint point, NSRect rect);
BOOL pointIsOnEdgeOfRect(NSPoint point, NSRect rect);
BOOL pointIsInRotatedRect(NSPoint point, NSRect rect, double rotationAngle, NSPoint rotationCenter); // the rotated rect should be specified as a non-rotated rect, with a rotation amount (in degrees with 0 at 12 o'clock and +ve moving clockwise) and a rotation center point

NSPoint pointInFlippedCoords(NSPoint point, NSRect enclosingBounds);
NSPoint nearestIntegralPoint(NSPoint point);
NSPoint pointByApplyingRotation(NSPoint point, double rotationAngle, NSPoint rotationCenter, BOOL clockwise);
NSPoint roundPoint(NSPoint point);
NSPoint floorPoint(NSPoint point);

NSSize sizeConstrainedToMinimumSize(NSSize size, NSSize minimums);
NSSize sizeConstrainedToMaximumSize(NSSize size, NSSize maximums);
NSSize sizeConstrainedToMaximumSideLengthPreservingAspectRatio(NSSize size, CGFloat maximumSideLength);


/* Dividing Rectangles into 9 areas */

typedef NS_ENUM(int, MWRectNineSplitArea) {
	undefinedArea = -1,
	bottomLeftArea = 0,
	bottomCenterArea,
	bottomRightArea,
	midLeftArea,
	midCenterArea,
	midRightArea,
	topLeftArea,
	topCenterArea,
	topRightArea
};

void nineSplitAreasForRect(NSRect rect, NSRectArray inout_splitAreasArray);
MWRectNineSplitArea nineSplitAreaForPointInRect(NSPoint point, NSRect rect);
BOOL isLeftColumn(MWRectNineSplitArea area);
BOOL isCenterColumn(MWRectNineSplitArea area);
BOOL isRightColumn(MWRectNineSplitArea area);
BOOL isBottomRow(MWRectNineSplitArea area);
BOOL isMidRow(MWRectNineSplitArea area);
BOOL isTopRow(MWRectNineSplitArea area);

