//
//  MWTextTokensView.h
//  Malkinware
//
//  Created by Jim McGowan on 15/3/13.
//  Copyright (c) 2013 Malkinware. All rights reserved.
//
//  NOTE: This view assumes it will be enclosed by an NSScrollView
//

#import <Cocoa/Cocoa.h>
#import "MWTokenView.h"

@class MWTokenListView;

@protocol MWTokenListViewDatasource;
@protocol MWTokenListViewDelegate;

typedef enum {HorizontalLayout, VerticalLayout} TokenListLayoutType;


@interface MWTokenListView : NSView

#pragma mark 10.7 Compatibility Code
// These can't be weak ref'd on 10.7 because they could bind to an NSWindowController
@property (assign) IBOutlet id <MWTokenListViewDelegate>   delegate;
@property (assign) IBOutlet id <MWTokenListViewDatasource> datasource;

@property (readonly) MWTokenView *selectedToken; 

@property (copy) NSColor *backgroundColor;
@property TokenListLayoutType tokenLayout;
@property CGFloat rowHeight;
@property CGFloat interRowSpacing;
@property CGFloat interTokenSpacing;
@property BOOL highlightsSelectedTokens;
@property BOOL preservesSelection;  //should try to maintain the selected token through data reloads
@property BOOL selectsInsertedTokens;
@property BOOL wraps; // if layout is horizontal, this property sets whether the view expands horizonally to accomodate a single row of tokens, or wraps at bounds width to new rows.

- (void)reloadData;

@end



@protocol MWTokenListViewDatasource <NSObject>

// Providing data for the view
- (NSUInteger)numberOfTokensForTokenListView:(MWTokenListView *)tokenListView;
- (NSAttributedString *)tokenListView:(MWTokenListView *)tokenListView displayStringForTokenAtIndex:(NSUInteger)tokenIndex;

// Optional additional data for tokens
@optional
- (id)tokenListView:(MWTokenListView *)tokenListView representedObjectForTokenAtIndex:(NSUInteger)tokenIndex;
- (NSColor *)tokenListView:(MWTokenListView *)tokenListView colorForTokenAtIndex:(NSUInteger)tokenIndex;
- (NSColor *)tokenListView:(MWTokenListView *)tokenListView selectedColorForTokenAtIndex:(NSUInteger)tokenIndex;
- (BOOL)tokenListView:(MWTokenListView *)tokenListView isHeaderTokenAtIndex:(NSUInteger)tokenIndex;

// Editing data in the view.
- (BOOL)tokenListView:(MWTokenListView *)tokenListView shouldEditDisplayStringOfTokenAtIndex:(NSUInteger)tokenIndex; // Assumes YES if not implemented by the datasource

//These methods should return YES if the datasource performs the edit, or NO to refuse the edit. If YES is returned, the view will call -reloadData on itself
@required
- (BOOL)tokenListView:(MWTokenListView *)tokenListView editedDisplayStringOfTokenAtIndex:(NSUInteger)tokenIndex to:(NSAttributedString *)editedString;
- (BOOL)tokenListView:(MWTokenListView *)tokenListView contentsOfPasteboard:(NSPasteboard *)pboard addedAtIndex:(NSUInteger)tokenIndex;
- (BOOL)tokenListView:(MWTokenListView *)tokenListView tokenMovedFromIndex:(NSUInteger)originalIndex toIndex:(NSUInteger)proposedIndex;
- (BOOL)tokenListView:(MWTokenListView *)tokenListView deletedTokenAtIndex:(NSUInteger)tokenIndex;
- (BOOL)tokenListView:(MWTokenListView *)tokenListView duplicatedTokenAtIndex:(NSUInteger)tokenIndex;

@end


@protocol MWTokenListViewDelegate <NSObject>

// Allowing paste
- (BOOL)tokenListView:(MWTokenListView *)tokenListView shouldAllowPasteFromPasteboard:(NSPasteboard *)pboard;

// Configuring Drag and Drop
- (BOOL)tokenListView:(MWTokenListView *)tokenListView shouldAllowDragOfTokenAtIndex:(NSUInteger)tokenIndex;
- (NSArray *)tokenListView:(MWTokenListView *)tokenListView UTIsForDraggedTokenAtIndex:(NSUInteger)tokenIndex;
- (NSDragOperation)sourceDragOperationMaskForTokenListView:(MWTokenListView *)tokenListView;
- (NSDragOperation)tokenListView:(MWTokenListView *)tokenListView destinationDragOperationForDraggingInfo:(id <NSDraggingInfo>)draggingInfo;

@end
